searchNodes=[{"doc":"Inspect values with syntax colors despite your remote console. This module addresses two surprises you&#39;ll encounter trying to dump data to the remote console like you did during development with iex -S mix: IO.inspect/1 et al work fine at the iex&gt; prompt but their output goes to the server&#39;s logs, not your console when you call them from inside a :telemetry handler function or other troubleshooting mechanism The syntax colors aren&#39;t working The inspection width is 80&hellip; just like iex, now that you think of it Why? See the explanation. In case of emergency, BREAK GLASS to get what you need with some copying and pasting. Usage To get the right syntax colors and inspection width, replace your calls to IO.inspect/1 with calls to Pretty.inspect/1: Pretty.inspect(&lt;&lt;0, 1, 2&gt;&gt;, width: 40) ... and the same for IO.inspect/2: [1, 2, 3] |&gt; IO.inspect(label: &quot;before&quot;) |&gt; Enum.map(&amp;(&amp;1 * 2)) |&gt; IO.inspect(label: &quot;after&quot;) |&gt; Enum.sum() To get the right colors, width, and output device, use Pretty.bind/1 to get an inspector/1 function, and use it instead of IO.inspect/1 or Pretty.inspect/1: dump = Pretty.bind(label: &quot;Ecto&quot;) handler = fn name, m10s, m6a, nil -&gt; dump.({name, m10s, m6a}) end :telemetry.attach(self(), [:my_app, :repo, :query], handler, nil)","ref":"Pretty.html","title":"Pretty","type":"module"},{"doc":"Bind an inspection function to the current standard output. See bind/2 for more details.","ref":"Pretty.html#bind/1","title":"Pretty.bind/1","type":"function"},{"doc":"Bind an inspection function to a particular device. The inspector&#39;s device and options are resolved when bind/2 is called, not when the inspector is called. If you bind at the remote console prompt, the device and options will remain appropriate for your console no matter which process calls the inspector. See inspect/3 for details on option handling.","ref":"Pretty.html#bind/2","title":"Pretty.bind/2","type":"function"},{"doc":"Inspect an item, writing the report to the standard output. Return the item unchanged. Like IO.inspect/2, but with pretty defaults appropriate to the device. See inspect/3 for details on option handling.","ref":"Pretty.html#inspect/2","title":"Pretty.inspect/2","type":"function"},{"doc":"Inspect an item, writing the report to a device. Return the item unchanged. Like IO.inspect/3, but with pretty defaults appropriate to the device: Set pretty to true Set width according to the device&#39;s :io.columns/1 if possible, else the Inspect.Opts default of 80 Set syntax_colors according to the device node&#39;s IEx.Config.inspect_opts/1","ref":"Pretty.html#inspect/3","title":"Pretty.inspect/3","type":"function"},{"doc":"Keyword options supported by IO.inspect/2.","ref":"Pretty.html#t:inspect_opts/0","title":"Pretty.inspect_opts/0","type":"type"},{"doc":"A 1-ary inspection function returning its argument unchanged e.g. IO.inspect/1.","ref":"Pretty.html#t:inspector/1","title":"Pretty.inspector/1","type":"type"},{"doc":"Explanation If you&#39;re sitting at a remote console right now and just need some output to your console from another process, BREAK GLASS. If not, read on:","ref":"explain.html","title":"Explanation","type":"extras"},{"doc":"Hooking up telemetry to Ecto.Adapters.SQL to watch your requests in action works from the iex -S mix phx.server prompt, but when you do it on a remote console you only see output for requests you made at the prompt: handler = fn name, m10s, m6a, nil -&gt; IO.inspect({name, m10s, m6a}, label: &quot;Ecto&quot;) end :telemetry.attach(self(), [:my_app, :repo, :query], handler, nil) To understand why, we need to look a little into how iex works at all. Your shell is a process, started by IEx.Server.run/1. It evaluates the code you type in another process called an evaluator running code starting with IEx.Evaluator.init/4. Input and output devices are also processes, sending and receiving messages according to the Erlang I/O Protocol. The Erlang documentation for :erlang.group_leader/0 says: Every process is a member of some process group and all groups have a group leader. All I/O from the group is channeled to the group leader. It doesn&#39;t say how the I/O from the group is channeled, but it&#39;s pretty simple: any function doing I/O will end up with some lower level function looking up your group leader and using it as the device if a specific device wasn&#39;t given. Sometimes this is direct: :io.put_chars/1 calls :erlang.group_leader/0 and gives it as the first argument to :io.put_chars/2. :io.request/1 similarly calls :io.request/2 with the group leader. Most of the time, though, it&#39;s indirect. I/O from almost everywhere else in OTP arrives at :io.request/2 (and, rarely, :io.requests/2) with an atom as its first argument. If that atom is :standard_io, it&#39;s replaced with the group leader. If not, the device is looked up by name in the process registry with :erlang.whereis/1. In Elixir, IO.puts/2 maps its device argument from :stdio to Erlang&#39;s :standard_io, :stderr to :standard_error, leaves other device arguments alone, and calls :io.put_chars/2 which proceeds as above. Given the above, these calls do the same thing: :io.put_chars(&#39;pretty!\\\\n&#39;) :io.put_chars(:erlang.group_leader(), &#39;pretty!\\\\n&#39;) :io.put_chars(:standard_io, &#39;pretty!\\\\n&#39;) IO.puts(&quot;pretty!&quot;) IO.puts(:stdio, &quot;pretty!&quot;) IO.puts(:standard_io, &quot;pretty!&quot;) IO.puts(Process.group_leader(), &quot;pretty!&quot;) Back to our &quot;missing&quot; output, consider what happens when you send I/O to various device aliases while your group leader is set to device other than your terminal. Paste this code into the iex prompt: test_output = fn device -&gt; IO.puts(device, [&quot;output to &quot;, Kernel.inspect(device)]) end run_experiments = fn -&gt; {:ok, dev_null} = File.open(&quot;/dev/null&quot;, [:read, :write]) group_leader = Process.group_leader() try do Process.group_leader(self(), dev_null) [:stdio, :standard_io, :stderr, :standard_error, :user] |&gt; Enum.map(fn device -&gt; {device, test_output.(device)} end) |&gt; Enum.into(%{}) after Process.group_leader(self(), group_leader) File.close(dev_null) end end run_experiments |&gt; Task.async() |&gt; Task.await() At the iex prompt, you&#39;ll see output for :stderr, :standard_error, and :user but not for :dev_null, stdio, or :standard_io despite the result showing IO.puts/2 returned :ok every time. The same is happening when Ecto.Adapters.SQL calls :telemetry.execute/3: Your telemetry handler is run in the same process that ran the query &mdash; often, it&#39;s handling the request end-to-end including Plug and Phoenix and Absinthe Your handler&#39;s call to IO.inspect/2 gets the default device :stdio That default gets resolved to the request&#39;s group leader Your output ends up in the node&#39;s standard output At the iex -S mix phx.server prompt, your shell is on the same node and you can&#39;t see the difference, setting up the surprise for when you use a remote shell To send the telemetry inspection to your remote console, capture your group leader outside the handler and write to it explicitly from inside the handler: device = Process.group_leader() handler = fn name, m10s, m6a, nil -&gt; IO.inspect(device, {name, m10s, m6a}, label: &quot;Ecto&quot;) end :telemetry.attach(self(), [:my_app, :repo, :query], handler, nil)","ref":"explain.html#the-case-of-the-missing-output","title":"Explanation - The Case of the Missing Output","type":"extras"},{"doc":"Trying to read pages of data structures without syntax coloring is hard. Where&#39;d the colors go? Let&#39;s check which nodes are running our shell and its evaluator when we run iex: iex&gt; self() |&gt; :erlang.node() :nonode@nohost iex&gt; :erlang.group_leader() |&gt; :erlang.node() :nonode@nohost ... and again when we run _build/dev/rel/my_app/bin/my_app remote: iex&gt; self() |&gt; :erlang.node() :&quot;my_app@127.0.0.1&quot; iex&gt; :erlang.group_leader() |&gt; :erlang.node() :&quot;rem-b987-my_app@127.0.0.1&quot; If the latter is less colorful than the former, it&#39;s because: Your remote shell called IEx.Config.inspect_opts/0 from a process on :&quot;my_app@127.0.0.1&quot;, not the hidden node running the other end of your shell IEx.Config.color/1 fell through to IO.ANSI.enabled?/0 because you didn&#39;t config :iex ANSI wasn&#39;t enabled because your application node had its output redirected at startup To put some color into your telemetry inspection, get the inspection options from your hidden node with :rpc.call/4. Extending our example above: device = Process.group_leader() opts = device |&gt; :erlang.node() |&gt; :rpc.call(IEx.Config, :inspect_opts, []) opts = Keyword.merge(opts, pretty: true, label: &quot;Ecto&quot;) handler = fn name, m10s, m6a, nil -&gt; IO.puts(device, [Kernel.inspect({name, m10s, m6a}, opts), IO.ANSI.reset()]) end :telemetry.attach(self(), [:my_app, :repo, :query], handler, nil) You&#39;ll need the IO.ANSI.reset/0 result to avoid the yellow reset handling numbers and functions for you also bleeding through to your prompt.","ref":"explain.html#the-case-of-the-missing-colors","title":"Explanation - The Case of the Missing Colors","type":"extras"},{"doc":"IEx.Config.width/0 returns the minimum of: Your console&#39;s width Application.get_env(:iex, :width, 80) To inspect at your console&#39;s full width, measure it outside your handler with io.columns/1. Extending our example again: device = Process.group_leader() opts = device |&gt; :erlang.node() |&gt; :rpc.call(IEx.Config, :inspect_opts, []) {:ok, width} = :io.columns(device) opts = Keyword.merge(opts, pretty: true, width: width, label: &quot;Ecto&quot;) handler = fn name, m10s, m6a, nil -&gt; IO.puts(device, [Kernel.inspect({name, m10s, m6a}, opts), IO.ANSI.reset()]) end :telemetry.attach(self(), [:my_app, :repo, :query], handler, nil) We&#39;ve done it! All three problems, solved&hellip; until you run them from a console for which io.columns/1 returns {:error, :enotsup}, or for which ANSI is genuinely not available and the reset will pollute your output. Go ahead, fix those; I&#39;ll wait right here. (Y&#39;know, you could cheat. Just sayin&#39;.)","ref":"explain.html#the-case-of-the-fixed-width","title":"Explanation - The Case of the Fixed Width","type":"extras"},{"doc":"To get the right colors, width, and output device, use Pretty.bind/1 to get a Pretty.inspector/1 function: dump = Pretty.bind(label: &quot;Ecto&quot;) Use that function instead of IO.inspect/1 or Pretty.inspect/1. Our Ecto.Adapters.SQL example becomes: dump = Pretty.bind(label: &quot;Ecto&quot;) handler = fn name, m10s, m6a, nil -&gt; dump.({name, m10s, m6a}) end :telemetry.attach(self(), [:my_app, :repo, :query], handler, nil) If you&#39;re in too much of a hurry to take a new dependency, though:","ref":"explain.html#the-case-of-great-relief","title":"Explanation - The Case of Great Relief","type":"extras"},{"doc":"Paste this in to get most of the functionality of Pretty.bind/1 right away: bind = fn opts -&gt; device = Process.group_leader() width = with {:ok, n} &lt;- :io.columns(device), do: n, else: (_ -&gt; %Inspect.Opts{}.width) opts = Keyword.merge(:rpc.call(:erlang.node(device), IEx.Config, :inspect_opts, []), opts) opts = Keyword.merge(opts, pretty: true, width: width) reset = Enum.find_value(Keyword.get(opts, :syntax_colors, []), [], fn _ -&gt; IO.ANSI.reset() end) fn term -&gt; IO.puts(device, [Kernel.inspect(term, opts), reset]); term end end # I never said it&#39;d be Pretty... # Can you make it shorter? PR or it didn&#39;t happen. ... and then use that: dump = bind.(label: &quot;ecto&quot;) handler = fn name, m10s, m6a, nil -&gt; dump.({name, m10s, m6a}) end :telemetry.attach(self(), [:my_app, :repo, :query], handler, nil)","ref":"explain.html#break-glass","title":"Explanation - BREAK GLASS","type":"extras"}]